package book;

import java.io.*;
import java.util.*;

public class RecordsBook {
    private Map<String, Record> _records = new TreeMap<String, Record>();
    private static final String RECORDS_FILE_NAME = "records.dat";

    public RecordsBook() {
        try {
            loadRecords();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @SuppressWarnings({"unchecked"})
    private void loadRecords() throws Exception {
        ObjectInputStream stream = null;
        try {
            stream = new ObjectInputStream(new FileInputStream(System.getProperty("user.home") + "/" + RECORDS_FILE_NAME));
            _records = (Map<String, Record>) stream.readObject();
        } catch (FileNotFoundException e) {
            // Ничего страшного, просто файла нет. Создадим новый.
        } finally {
            if (stream != null) {
                stream.close();
            }
        }
    }

    private void saveRecords() throws Exception {
        File file = new File(System.getProperty("user.home") + "/" + RECORDS_FILE_NAME);

        ObjectOutputStream stream = null;
        try {
            stream = new ObjectOutputStream(new FileOutputStream(file));
            stream.writeObject(_records);
        } finally {
            if (stream != null) {
                stream.close();
            }
        }
    }

    public void addRecord(Record aRecord) {
        _records.put(aRecord.getPhone(), aRecord);
        try {
            saveRecords();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void editRecord(String aOldPhone, Record aNewRecord) {
        _records.remove(aOldPhone);
        addRecord(aNewRecord);
    }

    public void removeRecord(String aPhone) {
        _records.remove(aPhone);
        try {
            saveRecords();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Collection<Record> getAllRecords() {
        return _records.values();
    }

    public String[][] getAllRecordsAsStrings() {
        String[][] result = new String[_records.size()][];

        int index = 0;
        for (Record record : _records.values()) {
            result[index] = new String[]{record.getPhone(), record.getName(), record.getComment()};
            index++;
        }

        return result;
    }
}
