package phoneBook.client;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.*;

public class PhoneBook implements EntryPoint {
    public void onModuleLoad() {
        final Button button = new Button("Add new record");
        RootPanel.get("buttonCell").add(button);

        final TextBox textPhone = new TextBox();
        final TextBox textName = new TextBox();
        final TextBox textComment = new TextBox();

        RootPanel.get("phoneCell").add(textPhone);
        RootPanel.get("nameCell").add(textName);
        RootPanel.get("commentCell").add(textComment);

        final FlexTable table = new FlexTable();

        RootPanel.get("main").add(table);

        button.addClickListener(new AddListener(textPhone, textName, textComment, table));

        PhoneBookService.App.getInstance().getRecords(new AsyncCallback() {
            public void onFailure(Throwable aThrowable) {
            }

            public void onSuccess(Object o) {
                String[][] strings = (String[][]) o;
                for (int i = 0; i < strings.length; i++) {
                    String[] string = strings[i];
                    setRow(table, string[0], string[1], string[2], table.getRowCount());
                }
            }
        });
    }

    private static class RemoveListener implements ClickListener {
        private FlexTable _table;
        private String _phone;

        public RemoveListener(String aPhone, FlexTable aTable) {
            _phone = aPhone;
            _table = aTable;
        }

        public void onClick(Widget aWidget) {
            PhoneBookService.App.getInstance().removeRecord(
                    _phone, new AsyncCallback() {
                public void onFailure(Throwable aThrowable) {
                    // ничего тут не будем делать
                }

                public void onSuccess(Object o) {
                    for (int i = 0; i < _table.getRowCount(); i++) {
                        if (_table.getText(i, 0).equals(_phone)) {
                            _table.removeRow(i);
                            break;
                        }
                    }
                }
            });
        }
    }

    private class EditListener implements ClickListener {
        private String _phone;
        private FlexTable _table;

        public EditListener(String aPhone, FlexTable aTable) {
            _phone = aPhone;
            _table = aTable;
        }

        public void onClick(Widget aWidget) {
            for (int i = 0; i < _table.getRowCount(); i++) {
                if (_table.getText(i, 0).equals(_phone)) {
                    final String phone = _table.getText(i, 0);
                    final int row = i;

                    final TextBox editPhone = new TextBox();
                    editPhone.setText(phone);

                    final TextBox editName = new TextBox();
                    editName.setText(_table.getText(row, 1));

                    final TextBox editComment = new TextBox();
                    editComment.setText(_table.getText(row, 2));

                    _table.setWidget(row, 0, editPhone);
                    _table.setWidget(row, 1, editName);
                    _table.setWidget(row, 2, editComment);

                    Button editButton = new Button("Update");
                    editButton.addClickListener(new ClickListener() {
                        public void onClick(Widget aWidget) {
                            PhoneBookService.App.getInstance().editRecord(
                                    phone, editPhone.getText(), editName.getText(), editComment.getText(), new AsyncCallback() {
                                public void onFailure(Throwable aThrowable) {
                                    // ничего тут не будем делать
                                }

                                public void onSuccess(Object o) {
                                    _table.removeRow(row);
                                    _table.insertRow(row);
                                    setRow(_table, editPhone.getText(), editName.getText(), editComment.getText(), row);
                                }
                            });
                        }
                    });

                    _table.setWidget(i, 3, editButton);
                    _table.removeCell(i, 4);
                }
            }
        }
    }

    private class AddListener implements ClickListener {
        private final TextBox _textPhone;
        private final TextBox _textName;
        private final TextBox _textComment;
        private final FlexTable _table;

        public AddListener(TextBox aTextPhone, TextBox aTextName, TextBox aTextComment, FlexTable aTable) {
            _textPhone = aTextPhone;
            _textName = aTextName;
            _textComment = aTextComment;
            _table = aTable;
        }

        public void onClick(Widget aWidget) {
            PhoneBookService.App.getInstance().addRecord(
                    _textPhone.getText(),
                    _textName.getText(),
                    _textComment.getText(), new AsyncCallback() {
                public void onFailure(Throwable aThrowable) {
                    // ничего тут не будем делать
                }

                public void onSuccess(Object o) {
                    String phone = _textPhone.getText();
                    String name = _textName.getText();
                    String comment = _textComment.getText();

                    setRow(_table, phone, name, comment, _table.getRowCount());
                }
            });
        }
    }

    private void setRow(FlexTable aTable, String aPhone, String aName, String aComment, int aRow) {
        aTable.setText(aRow, 0, aPhone);
        aTable.setText(aRow, 1, aName);
        aTable.setText(aRow, 2, aComment);

        Button buttonRemove = new Button("Remove");
        buttonRemove.addClickListener(new RemoveListener(aPhone, aTable));
        aTable.setWidget(aRow, 3, buttonRemove);

        Button buttonEdit = new Button("Edit");
        buttonEdit.addClickListener(new EditListener(aPhone, aTable));
        aTable.setWidget(aRow, 4, buttonEdit);
    }
}
